@@ -19,6 +19,7 @@ from sqlalchemy.types import (
     Unicode,
 )
 from sqlalchemy.sql.schema import ForeignKeyConstraint
+from codalab.common import StorageType
 
 db_metadata = MetaData()
 
@@ -41,6 +42,15 @@ bundle = Table(
     Column('state', String(63), nullable=False),
     Column('owner_id', String(255), nullable=True),
     Column('is_anonymous', Boolean, nullable=False, default=False),
+    Column(
+        'storage_type',
+        Enum(StorageType.DISK_STORAGE.value, StorageType.AZURE_BLOB_STORAGE.value),
+        nullable=True,
+    ),  # Where the bundle contents are stored. If set to null, nothing has been uploaded for the bundle yet.
+    # When updating this column, sync it with codalab.common.StorageType.
+    Column(
+        'is_dir', Boolean, nullable=True,
+    ),  # Whether the bundle is a directory or just a single file. If set to null, nothing has been uploaded for the bundle yet.
     UniqueConstraint('uuid', name='uix_1'),
     Index('bundle_data_hash_index', 'data_hash'),
     Index('state_index', 'state'),  # Needed for the bundle manager.
@@ -283,6 +293,9 @@ user = Table(
     Column('time_used', Float, nullable=False),  # Number of seconds already used
     Column('disk_quota', Float, nullable=False),  # Number of bytes allowed
     Column('disk_used', Float, nullable=False),  # Number of bytes already used
+    Column(
+        'avatar_id', String(63), nullable=True
+    ),  # bundle id of the user's uploaded profile picture; Null if the user has never uploaded one
     Index('user_user_id_index', 'user_id'),
     Index('user_user_name_index', 'user_name'),
     UniqueConstraint('user_id', name='uix_1'),
