@@ -197,6 +197,8 @@ class BundleSchema(Schema):
     state = fields.String()
     owner = fields.Relationship(include_resource_linkage=True, type_='users', attribute='owner_id')
     is_anonymous = fields.Bool()
+    storage_type = fields.String()
+    is_dir = fields.Bool()
     metadata = fields.Dict(values=fields.String(validate=validate_ascii))
     dependencies = fields.Nested(BundleDependencySchema, many=True)
     children = fields.Relationship(
@@ -268,6 +270,7 @@ class UserSchema(Schema):
     affiliation = fields.String(allow_none=True)
     url = fields.Url(allow_none=True)
     date_joined = fields.LocalDateTime("%c")
+    avatar_id = fields.String(allow_none=True)
 
     class Meta:
         type_ = 'users'
