@@ -150,12 +150,7 @@ class CodalabArg(object):
 
 CODALAB_ARGUMENTS = [
     # Basic settings
-    CodalabArg(
-        name='version',
-        help='Version of CodaLab (usually the branch name)',
-        default=get_default_version(),
-        flag='-v',
-    ),
+    CodalabArg(name='version', help='Version of CodaLab (usually the branch name)', flag='-v',),
     CodalabArg(
         name='instance_name',
         help='Instance name (prefixed to Docker containers)',
@@ -358,6 +353,11 @@ CODALAB_ARGUMENTS = [
         help='Comma-separated list of directories that are mounted on the REST server, allowing their contents to be used in the --link argument.',
         default='/tmp/codalab/link-mounts',
     ),
+    CodalabArg(name='azure_blob_connection_string', help='Azure Blob storage connection string'),
+    CodalabArg(
+        name='always_use_azure_blob_beta',
+        help='If set, Azure Blob Storage is always used to store uploads, regardless of the use_azure_blob_beta parameter.',
+    ),
     # Public workers
     CodalabArg(name='public_workers', help='Comma-separated list of worker ids to monitor'),
 ]
@@ -612,6 +612,8 @@ class CodalabServiceManager(object):
 
         if self.args.version:
             self.args.version = clean_version(self.args.version)
+        else:
+            self.args.version = get_default_version()
         self.compose_cwd = os.path.join(BASE_DIR, 'docker_config', 'compose_files')
 
         self.compose_files = []
